

set -ex



test -f $PREFIX/include/gsasl-compat.h
test -f $PREFIX/include/gsasl-mech.h
test -f $PREFIX/include/gsasl.h
test -f $PREFIX/lib/libgsasl.a
test -f $PREFIX/lib/libgsasl.so
test -f $PREFIX/lib/libgsasl.so.7
test -f $PREFIX/lib/libgsasl.so.7.10.0
test -f $PREFIX/lib/libgsasl$SHLIB_EXT
exit 0
